/*
 * Copyright 2006-2007 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.openspaces.core.context;

import org.openspaces.core.GigaSpace;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Allows to directly inject a {@link GigaSpace} implementation into a class field or setter
 * property. A name can be specified in cases where more than one {@link GigaSpace} are defined
 * within a spring application context. The name will be the bean name / id.
 * 
 * @author kimchy
 */
@Target( { ElementType.METHOD, ElementType.FIELD })
@Retention(RetentionPolicy.RUNTIME)
public @interface GigaSpaceContext {

    /**
     * The name of the {@link GigaSpace} bean. Used when more than one {@link GigaSpace} is defined
     * and corresponds to the bean name / id.
     */
    String name() default "";
}
